"""Databases tools"""

from typing import *

from pymongo import MongoClient
from pymongo.database import Database
from pymongo.collection import Collection
from pymongo.errors import DuplicateKeyError
from pymongo.results import InsertOneResult, InsertManyResult

from loguru import logger


def create_collection(db: Database, collection: str, keys: Optional[List[Tuple[str, int]]]=None, index: Optional[Any] = None) -> Collection:
    """创建集合
    
    :param db: mongo的db对象
    :param collection: 集合名称
    :param keys: 主键索引，支持联合主键，格式（列表元组或列表列表均可）：[["id", 1], ["user.id", 1], ["created_at", 1]]
    :parma index: 索引，可以是单个字符，也可以是列表，目前只支持单个索引创建
    """
    if collection not in db.list_collection_names():
        coll = db.create_collection(collection)
        coll.create_index(keys, unique=True)
        coll.create_index(index)
    else:
        coll = db.get_collection(collection)
    return coll


def set_results(collection: Collection, datas: List[Dict], if_exists='ignore') -> int:
    """插入数据

    :param collection:
    :param datas:
    :param if_exists: 重复处理，包括：'ignore' 'replace' 
    :return:
    """
    if not isinstance(datas, (list, tuple, set)):
        datas = [datas]

    add_count = 0
    for data in datas:
        try:
            insert_res = collection.insert_one(data)
            _id = insert_res.inserted_id
        except DuplicateKeyError as e:
            if if_exists == 'ignore':
                continue
            else:
                # 解析错误信息以获取重复的键值
                error_details = e.details
                logger.debug(f"捕获到重复键错误: {error_details}")
                if 'keyValue' in error_details:
                    # 获取重复的键值
                    duplicate_key_value = error_details['keyValue']
                    # print(duplicate_key_value, data)
                    duplicate_key_value.pop("_id", None)
                    data.pop('_id', None)
                    logger.debug(f"重复的键值: {duplicate_key_value}")

                    # 使用获取到的键值进行更新操作
                    update_result = collection.update_one(
                        duplicate_key_value,
                        {'$set': data},
                        upsert=True
                    )
                    if update_result.upserted_id:
                        logger.debug(f"文档不存在，已插入，文档ID: {update_result.upserted_id}")
                    else:
                        logger.debug(f"文档已存在，已更新")
        add_count += 1
    return add_count



import os
from datetime import datetime, timedelta
from typing import *
import pandas as pd
import numpy as np

from jtools import consts as C


SH_STARTDATE, SH_ENDDATE = '19901219', '20991231'


def _wind_code2inst_id(wind_codes: List[str]) -> List[str]:
    """转换：wind_code（三位交易所代码）转inst_id（二位交易所代码）"""
    inst_ids = {}
    for wind_code in wind_codes:
        inst_name, wexchg_id = wind_code.split('.')
        exchg_id = C.MAP_WEXCHG_XTEXCHG.get(wexchg_id, wexchg_id)
        
        if exchg_id not in ['CZC', 'CFE']:
            inst_name = inst_name.lower()
        inst_id = '.'.join([inst_name, exchg_id])
        # wind_codes.append()
        inst_ids[wind_code] = inst_id
    return inst_ids


def to_inst_id(symbols: List[str], source='wind_code') -> List[str]:
    """转换：转为inst_id（二位交易所代码）"""
    if source == 'wind_code':
        return _wind_code2inst_id(symbols)
    else:
        raise ValueError()
    

def get_trading_dates(market, startdate: str, enddate: str) -> List[str]:
    """获取交易日"""
    startdate, enddate = startdate.replace('-', ''), enddate.replace('-', '')
    today = datetime.today()
    # enddate = min(today.strftime("%Y%m%d"), enddate)
    enddate = min(enddate, today.strftime("%Y1231")) # 改为当年最后一日（一般holiday是一年一次更新）
    
    exdates = C.HOLIDAYS.get(market, 'SH')
    if startdate < "20241231":
        fp_his = os.path.join(os.path.dirname(__file__), f'data/trddt_{market}.csv')
        # fp_his = f'data/trddt_{market}.csv'
        all_hisdates = pd.read_csv(fp_his, header=None, dtype=str)[0]
        all_hisdates = all_hisdates[(all_hisdates>=startdate) & (all_hisdates<=enddate)].values.tolist()
    else:
        # 起点超过历史范围：直接使用最新生成数据
        all_hisdates = []
    if enddate > '20241231':
        all_bdates = pd.bdate_range(start=max(startdate, "20241231"), end=enddate).strftime("%Y%m%d").tolist()
    else:
        # 起止都在历史范围，直接调用历史数据
        all_bdates = []
    all_trddts = list(sorted(set(all_hisdates + all_bdates) - set(exdates)))
    return all_trddts


def get_last_trddt(market='SH', n=0, enddate=None) -> str:
    """获取：上一个成交日
    
    - 当日非交易日，默认返回前一个交易日
    - 当日交易日，返回当日
    :param n: 多少个交易日
    :param enddate: 从哪一天开始的上一个交易日
    :return: trddt in %Y%m%d format
    """
    if enddate is None:
        _now = datetime.today()
    else:
        _now = datetime.strptime(enddate, "%Y%m%d")
    _n = max(30, n) * 2
    _stdate = (_now - timedelta(days=_n)).strftime("%Y%m%d")
    _trddts = get_trading_dates(market, _stdate, _now.strftime("%Y%m%d"))
    return _trddts[-n-1]


def get_next_trddt(startdate=None, n=0, market="SH"):
    """获取：下一个交易日（含当日）
    - 当日非交易日，默认返回下一个交易日
    - 当日交易日，返回当日
    :param startdate: 从哪一天开始的上一个交易日
    :param n: 多少个交易日
    :return: trddt in %Y%m%d format
    """
    if startdate is None:
        _stdate = datetime.today()
    else:
        _stdate = datetime.strptime(startdate, "%Y%m%d")
    _n = max(30, n) * 2
    _eddate = (_stdate + timedelta(days=_n)).strftime("%Y%m%d")
    _trddts = get_trading_dates(market, _stdate.strftime("%Y%m%d"), _eddate)
    return _trddts[n]


def get_latest_trddt(market='SH') -> str:
    """获取：最近交易日

    - 当日交易日，返回当日
    - 当日非交易日，默认返回下一个交易日
    """
    _now = datetime.today()
    _eddate = (_now + timedelta(days=30)).strftime("%Y%m%d")
    _trddts = get_trading_dates(market, _now.strftime("%Y%m%d"), _eddate)
    return _trddts[0]


def is_trading_date(trddt, market='SH') -> bool:
    """判断：是否是交易日
    
    :param trddt: 交易日，支持：datetime(2025, 8, 25), '20250825', '2025-08-25', 20250825, 1747929600000, 1747929600
    :param market: 市场，默认："SH"上交所
    """
    if isinstance(trddt, str):
        trddt = trddt.replace('-', '')
    elif isinstance(trddt, (float, int)):
        if trddt > 1e10:
            trddt /= 1e3  # 转为秒
        trddt = datetime.fromtimestamp(trddt).strftime("%Y%m%d")
    else:
        assert isinstance(trddt, datetime)
        trddt = trddt.strftime("%Y%m%d")
    
    if trddt in CACHE_TRDDTS:
        return True
    else:
        return False


CACHE_TRDDTS = get_trading_dates(market='SH', startdate=SH_STARTDATE, enddate=SH_ENDDATE)




# setup.py
from setuptools import setup, find_packages
from jtools import __name__, __version__

setup(
    name="dsf-" + __name__,
    version=__version__,
    author='DSFish',
    author_email='liumingshuo1017@gmail.com',
    description='A simple tools module',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/DataScienceFish/jtools',
    packages=find_packages(),
    package_data={
        'jtools': ['data/*.csv'],  # 包含 jtools/data 文件夹中的所有 CSV 文件
    },
    install_requires=[
        "pymongo==4.11.2",  # 指定 pymongo 的版本
        "loguru==0.7.2",  # loguru
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.9.12',
)